!        routine name   - icsccu
!
!-----------------------------------------------------------------------
!
!   purpose             - cubic spline interpolation
!                           (easy-to-use version)
!
!   usage               - call icsccu (x,y,nx,c,ic,ier)
!
!   arguments    x      - vector of length nx containing the abscissae
!                           of the nx data points (x(i),y(i)) i=1,...,
!                           nx. (input) x must be ordered so that
!                           x(i) .lt. x(i+1).
!                y      - vector of length nx containing the ordinates
!                           (or function values) of the nx data points.
!                           (input)
!                nx     - number of elements in x and y. (input) nx
!                           must be .ge. 2.
!                c      - spline coefficients. (output) c is an nx-1 by
!                           3 matrix. the value of the spline
!                           approximation at t is
!                           s(t) = ((c(i,3)*d+c(i,2))*d+c(i,1))*d+y(i)
!                           where x(i) .le. t .lt. x(i+1) and
!                           d = t-x(i).
!                ic     - row dimension of matrix c exactly as
!                           specified in the dimension statement in
!                           the calling program. (input)
!                ier    - error parameter. (output)
!                         terminal error
!                           ier = 129, ic is less than nx-1.
!                           ier = 130, nx is less than 2.
!                           ier = 131, input abscissa are not ordered
!                             so that x(1) .lt. x(2) ... .lt. x(nx).
!
!
!-----------------------------------------------------------------------
!
      subroutine icsccu (x,y,nx,c,ic,ier)

!              specifications for arguments
      integer            nx,ic,ier
      double precision   x(nx),y(nx),c(ic,3)
!              specifications for local variables
      integer            im1,i,jj,j,mm1,mp1,m,nm1,nm2
      double precision   divdf1,divdf3,dtau,g,cnx(3)
!              first executable statement
      nm1 = nx-1
      ier = 129
      if (ic .lt. nm1) go to 9000
      ier = 130
      if (nx .lt. 2) go to 9000
      ier = 131
      if (nx .eq. 2) go to 45
!               compute not-a-knot spline
      do 5 m = 2,nm1
         mm1=m-1
         c(m,2) = x(m)-x(mm1)
         if (c(m,2).le.0.0d0) go to 9000
         c(m,3) = (y(m)-y(mm1))/c(m,2)
    5 continue
      cnx(2) = x(nx)-x(nm1)
      if (cnx(2).le.0.0d0) go to 9000
      cnx(3) = (y(nx)-y(nm1))/cnx(2)
      ier = 0
      nm2 = nx-2
      if (nx .gt. 3) go to 10
      c(1,3) = cnx(2)
      c(1,2) = c(2,2)+cnx(2)
      c(1,1) = ((c(2,2)+2.d0*c(1,2))*c(2,3)*cnx(2)+c(2,2)**2*cnx(3))   &
     &/c(1,2)
      go to 20
   10 c(1,3) = c(3,2)
      c(1,2) = c(2,2)+c(3,2)
      c(1,1) = ((c(2,2)+2.d0*c(1,2))*c(2,3)*c(3,2)+c(2,2)**2*c(3,3))   &
     &/c(1,2)
      do 15 m=2,nm2
         mp1=m+1
         mm1=m-1
         g = -c(mp1,2)/c(mm1,3)
         c(m,1) = g*c(mm1,1)+3.d0*c(m,2)*c(mp1,3)+3.d0*c(mp1,2)*c(m,3)
         c(m,3) = g*c(mm1,2)+2.d0*c(m,2)+2.d0*c(mp1,2)
   15 continue
   20 g = -cnx(2)/c(nm2,3)
      c(nm1,1) = g*c(nm2,1)+3.d0*c(nm1,2)*cnx(3)+3.d0*cnx(2)*c(nm1,3)
      c(nm1,3) = g*c(nm2,2)+2.d0*c(nm1,2)+2.d0*cnx(2)
      if (nx.gt.3) go to 25
      cnx(1)=2.d0*cnx(3)
      cnx(3)=1.d0
      g=-1.d0/c(nm1,3)
      go to 30
   25 g = c(nm1,2)+cnx(2)
      cnx(1) = ((cnx(2)+2.d0*g)*cnx(3)*c(nm1,2)+cnx(2)**2*    &
     &      (y(nm1)-y(nx-2))/c(nm1,2))/g
      g = -g/c(nm1,3)
      cnx(3) = c(nm1,2)
   30 cnx(3) = g*c(nm1,2)+cnx(3)
      cnx(1) = (g*c(nm1,1)+cnx(1))/cnx(3)
      c(nm1,1) = (c(nm1,1)-c(nm1,2)*cnx(1))/c(nm1,3)
      do 35 jj=1,nm2
         j = nm1-jj
         c(j,1) = (c(j,1)-c(j,2)*c(j+1,1))/c(j,3)
   35 continue
      do 40 i=2,nm1
         im1 = i-1
         dtau = c(i,2)
         divdf1 = (y(i)-y(im1))/dtau
         divdf3 = c(im1,1)+c(i,1)-2.d0*divdf1
         c(im1,2) = (divdf1-c(im1,1)-divdf3)/dtau
         c(im1,3) = divdf3/dtau**2
   40 continue
      dtau = cnx(2)
      divdf1 = (y(nx)-y(nm1))/dtau
      divdf3 = c(nm1,1)+cnx(1)-2.d0*divdf1
      c(nm1,2) = (divdf1-c(nm1,1)-divdf3)/dtau
      c(nm1,3) = divdf3/dtau**2
      go to 9005
   45 if (x(1) .ge. x(2)) go to 9000
      ier = 0
      c(1,1) = (y(2)-y(1))/(x(2)-x(1))
      c(1,2) = 0.0d0
      c(1,3) = 0.0d0
      go to 9005
 9000 continue
      write (6,10000) ier 
10000 format(1x,' Warning:  ier=',i5)
 9005 return
      end
!         routine name   - icsevu
!
!-----------------------------------------------------------------------
!
!   purpose             - evaluation of a cubic spline
!
!   usage               - call icsevu(x,y,nx,c,ic,u,s,m,ier)
!
!   arguments    x      - vector of length nx containing the abscissae
!                           of the nx data points (x(i),y(i)) i=1,...,
!                           nx (input). x must be ordered so that
!                           x(i) .lt. x(i+1).
!                y      - vector of length nx containing the ordinates
!                           (or function values) of the nx data points
!                           (input).
!                nx     - number of elements in x and y (input).
!                           nx must be .ge. 2.
!                c      - spline coefficients (input). c is an nx-1 by
!                           3 matrix.
!                ic     - row dimension of matrix c exactly as
!                           specified in the dimension statement
!                           in the calling program (input).
!                           ic must be .ge. nx-1.
!                u      - vector of length m containing the abscissae
!                           of the m points at which the cubic spline
!                           is to be evaluated (input).
!                s      - vector of length m (output).
!                           the value of the spline approximation at
!                           u(i) is
!                           s(i) = ((c(j,3)*d+c(j,2))*d+c(j,1))*d+y(j)
!                           where x(j) .le. u(i) .lt. x(j+1) and
!                           d = u(i)-x(j).
!                m      - number of elements in u and s (input).
!                ier    - error parameter (output).
!                         warning error
!                           ier = 33, u(i) is less than x(1).
!                           ier = 34, u(i) is greater than x(nx).
!
!
!   remarks  1.  the routine assumes that the abscissae of the nx
!                data points are ordered such that x(i) is less than
!                x(i+1) for i=1,...,nx-1. no check of this condition
!                is made in the routine. unordered abscissae will cause
!                the algorithm to produce incorrect results.
!            2.  the routine generates two warning errors. one error
!                occurs if u(i) is less than x(1), for some i in the
!                the interval (1,m) inclusively. the other error occurs
!                if u(i) is greater than x(nx), for some i in the
!                interval (1,m) inclusively.
!            3.  the ordinate y(nx) is not used by the routine. for
!                u(k) .gt. x(nx-1), the value of the spline, s(k), is
!                given by
!                 s(k)=((c(nx-1,3)*d+c(nx-1,2))*d+c(nx-1,1))*d+y(nx-1)
!                where d=u(k)-x(nx-1).
!
!
!-----------------------------------------------------------------------
!
      subroutine icsevu  (x,y,nx,c,ic,u,s,m,ier)

!             specifications for arguments
      integer            nx,ic,m,ier
      double precision   x(nx),y(nx),c(ic,3),u(m),s(m)
!             specifications for local variables
      integer            i,jer,ker,nxm1,k
      double precision   d,dd,zero
      data               i/1/,zero/0.0d0/
!             first executable statement
      jer = 0
      ker = 0
      if (m .le. 0) go to 9005
      nxm1 = nx-1
      if (i .gt. nxm1) i = 1
!             evaluate spline at m points
      do 40 k=1,m
!             find the proper interval
         d = u(k)-x(i)
         if (d) 5,25,15
    5    if (i .eq. 1) go to 30
         i = i-1
         d = u(k)-x(i)
         if (d) 5,25,20
   10    i = i+1
         d = dd
   15    if (i .ge. nx) go to 35
         dd = u(k)-x(i+1)
         if (dd .ge. zero) go to 10
         if (d .eq. zero) go to 25
!              perform evaluation
   20    s(k) = ((c(i,3)*d+c(i,2))*d+c(i,1))*d+y(i)
         go to 40
   25    s(k) = y(i)
         go to 40
!              warning - u(i) .lt. x(1)
   30    jer = 33
         go to 20
!              if u(i) .gt. x(nx) - warning
   35    if (dd .gt. zero) ker = 34
         d = u(k)-x(nxm1)
         i = nxm1
         go to 20
   40 continue
      ier = max(jer,ker)
 9000 continue
      if (jer .gt. 0) write (6,*) 'warning: jer=',jer 
      if (ker .gt. 0) write (6,*) 'warning: ker=',ker 
 9005 return
      end
